import { selectAll } from 'unist-util-select';
import { toText } from '../utils.js';
/**
 * Attempt to pull abbreviations out of tree
 *
 * This looks for parenthesized letters and tries to match them to the
 * previous words.
 */
export function abbreviationsFromTree(tree, frontmatter) {
    let abbreviations = {};
    const paragraphs = selectAll('paragraph', tree);
    paragraphs.forEach((paragraph) => {
        const text = toText(paragraph);
        abbreviations = {
            ...abbreviations,
            ...abbreviationsFromText(text),
        };
    });
    frontmatter.abbreviations = { ...frontmatter.abbreviations, ...abbreviations };
}
function maybeStopWord(word) {
    return word.length < 5;
}
function exploreAbbrPossibilities(letter, possibilities) {
    const newPossibilities = [];
    possibilities.forEach(({ prev, next }) => {
        if (prev === null || prev === void 0 ? void 0 : prev.includes(letter)) {
            newPossibilities.push({
                prev: prev.slice(prev.indexOf(letter) + 1),
                next,
            });
        }
        for (const [i, n] of next.entries()) {
            if (n.startsWith(letter)) {
                newPossibilities.push({
                    prev: next[i].slice(1),
                    next: next.slice(i + 1),
                });
            }
            if (!prev || !maybeStopWord(n)) {
                break;
            }
        }
    });
    return newPossibilities;
}
export function abbreviationsFromText(text) {
    const abbreviations = {};
    const textList = text.split(' ');
    textList.forEach((word, index) => {
        var _a;
        const abbr = (_a = word.match(/^\(([^\s]{2,})\).{0,1}/)) === null || _a === void 0 ? void 0 : _a[1];
        if (!abbr)
            return;
        const possibleWords = [];
        let wordIndex = index - 1;
        while (textList[wordIndex] && possibleWords.filter((w) => w.length > 4).length < abbr.length) {
            possibleWords.unshift(textList[wordIndex]);
            wordIndex--;
        }
        for (const i of Array(possibleWords.length).keys()) {
            let possibilities = [
                {
                    next: possibleWords.slice(i).map((w) => w.toLowerCase()),
                },
            ];
            abbr
                .split('')
                .filter((letter) => letter.match(/^[a-zA-Z]$/))
                .forEach((letter) => {
                possibilities = exploreAbbrPossibilities(letter.toLowerCase(), possibilities);
            });
            if (possibilities.filter(({ next }) => next.length === 0).length) {
                abbreviations[abbr] = possibleWords.slice(i).join(' ');
                break;
            }
        }
    });
    return abbreviations;
}
