import { RuleId, fileWarn } from 'myst-common';
import { select, selectAll } from 'unist-util-select';
import { remove } from 'unist-util-remove';
import { Tags } from 'jats-tags';
/**
 * Convert boxed-text caption to admonitionTitle (with content from caption > title)
 *
 * If boxed-text caption does not have a title, warn.
 *
 * After converting, delete any remaining boxed-text captions.
 */
export function admonitionTransform(tree, file) {
    const captions = selectAll(`${Tags.boxedText} > ${Tags.caption}`, tree);
    captions.forEach((caption) => {
        const title = select(Tags.title, caption);
        if (!title) {
            fileWarn(file, '', { node: caption, ruleId: RuleId.jatsParses });
            return;
        }
        caption.type = 'admonitionTitle';
        caption.children = title.children;
    });
    const noCaptions = selectAll(`${Tags.boxedText} > ${Tags.caption}`, tree);
    noCaptions.forEach((caption) => {
        caption.type = '__delete__';
    });
    remove(tree, '__delete__');
}
export const admonitionPlugin = () => (tree, file) => {
    admonitionTransform(tree, file);
};
