import { copyNode } from 'myst-common';
import { selectAll } from 'unist-util-select';
/**
 * Copy footnotes and sections from back into body tree
 *
 * Back is not modified.
 */
export function backToBodyTransform(body, back) {
    var _a;
    if (!back)
        return;
    const backNodes = (_a = back.children) === null || _a === void 0 ? void 0 : _a.filter((node) => {
        return ['fn-group', 'sec', 'ack', 'app-group'].includes(node.type);
    });
    if (!(body === null || body === void 0 ? void 0 : body.children) || backNodes.length === 0)
        return;
    body.children.push({ type: 'hr' }, ...copyNode(backNodes));
}
/**
 * Leave table footnotes in legend if they are not referenced anywhere
 */
export function tableFootnotesToLegend(tree) {
    const tableFns = selectAll('legend > footnoteDefinition', tree);
    const fnRefs = selectAll('footnoteReference', tree).map(({ identifier }) => identifier);
    tableFns.forEach((tableFn) => {
        if (tableFn.identifier && fnRefs.includes(tableFn.identifier))
            return;
        tableFn.type = 'paragraph';
    });
}
