import { liftChildren, toText } from 'myst-common';
import { blockNestingTransform } from 'myst-transforms';
import { select, selectAll } from 'unist-util-select';
import { remove } from 'unist-util-remove';
const SECTION_TYPES = ['sec', 'ack', 'app'];
function isSection(node) {
    return SECTION_TYPES.includes(node.type);
}
function liftApps(tree) {
    selectAll('app-group', tree).forEach((group) => {
        group.type = '__lift__';
    });
    liftChildren(tree, '__lift__');
}
function recurseSections(tree, depth = 1, titleType) {
    var _a;
    const sections = (_a = tree.children) === null || _a === void 0 ? void 0 : _a.filter((n) => isSection(n));
    if (!sections || sections.length === 0)
        return;
    sections.forEach((sec) => {
        var _a, _b;
        let firstChild = (_a = sec.children) === null || _a === void 0 ? void 0 : _a[0];
        // Section labels are ignored
        if ((firstChild === null || firstChild === void 0 ? void 0 : firstChild.type) === 'label') {
            firstChild.type = '__delete__';
            firstChild = (_b = sec.children) === null || _b === void 0 ? void 0 : _b[1];
        }
        if ((firstChild === null || firstChild === void 0 ? void 0 : firstChild.type) === 'title') {
            if (sec.type === 'ack' && toText(firstChild).toLowerCase().startsWith('ack')) {
                firstChild.type = '__delete__';
            }
            else if (titleType === 'strong') {
                firstChild.type = 'p';
                firstChild.children = [{ type: 'bold', children: firstChild.children }];
            }
            else {
                firstChild.type = 'heading';
                firstChild.id = sec.id;
                firstChild.depth = depth;
            }
        }
        if (sec.type === 'ack')
            sec.part = 'acknowledgments';
        if (sec.type === 'app')
            sec.part = 'appendix';
        sec.type = 'sec';
        recurseSections(sec, depth + 1, titleType);
    });
}
/**
 * Clean up nested sections
 *
 * - Identify sections with titles
 * - Convert those titles to headings
 * - Give headings depth value based on nesting
 * - Flatten the sections
 */
export function sectionTransform(tree, titleType) {
    var _a;
    liftApps(tree);
    recurseSections(tree, 1, titleType);
    remove(tree, '__delete__');
    const topSections = (_a = tree.children) === null || _a === void 0 ? void 0 : _a.filter((n) => isSection(n));
    topSections.forEach((sec) => {
        sec.type = 'block';
    });
    while (select('sec', tree))
        liftChildren(tree, 'sec');
    blockNestingTransform(tree);
}
export const sectionPlugin = () => (tree) => {
    sectionTransform(tree);
};
